﻿/*
 * Created by SharpDevelop.
 * User: misthema
 * Date: 11.9.2017
 * Time: 17:06
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using TradeServer.TradeLogicDriver.DataObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace ExampleNameSpace
{
	/// <summary>
	/// Tenkan-sen (Conversion Line): (9-period high + 9-period low)/2)) 
	/// The default setting is 9 periods and can be adjusted. On a daily 
	/// chart, this line is the mid point of the 9 day high-low range, 
	/// which is almost two weeks.  
	/// 
	/// Kijun-sen (Base Line): (26-period high + 26-period low)/2)) 
	/// The default setting is 26 periods and can be adjusted. On a daily 
	/// chart, this line is the mid point of the 26 day high-low range, 
	/// which is almost one month).  
	/// 
	/// Senkou Span A (Leading Span A): (Conversion Line + Base Line)/2)) 
	/// This is the midpoint between the Conversion Line and the Base Line. 
	/// The Leading Span A forms one of the two Cloud boundaries. It is 
	/// referred to as "Leading" because it is plotted 26 periods in the future
	/// and forms the faster Cloud boundary. 
	/// 
	/// Senkou Span B (Leading Span B): (52-period high + 52-period low)/2)) 
	/// On the daily chart, this line is the mid point of the 52 day high-low range, 
	/// which is a little less than 3 months. The default calculation setting is 
	/// 52 periods, but can be adjusted. This value is plotted 26 periods in the future 
	/// and forms the slower Cloud boundary.
	/// 
	/// Chikou Span (Lagging Span): Close plotted 26 days in the past
	/// The default setting is 26 periods, but can be adjusted.
	/// 
	/// @: http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
	/// </summary>
    public class IchimokuCloudsExt : IIndicatorScript
    {    	
    	const string WEAK = "weak";
    	const string NEUTRAL = "neutral";
    	const string STRONG = "strong";
    	
    	int tenkansen = 20, kijunsen = 60, senkouspanb = 120, chikouspan = 30;
    	string buySignalStrength = NEUTRAL;
		string sellSignalStrength = NEUTRAL;
    	    	
    	double convLine, baseLine, leadingSpanA, leadingSpanB, laggingSpan;

        // Properties
        public string Name
        {
            get {
        		return "Ichimoku Clouds Enhanced";
        	}
        }
		
		public int TicksBack {
			get {
				return senkouspanb + chikouspan + 5;				
			}
		}

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> res = new List<ScriptParameter>
            {
                new ScriptParameter("Tenkan Sen", ScriptParameterType.Integer, tenkansen.ToString()),
                new ScriptParameter("Kijun Sen", ScriptParameterType.Integer, kijunsen.ToString()),
                new ScriptParameter("Senkou Span B", ScriptParameterType.Integer, senkouspanb.ToString()),
                new ScriptParameter("Chikou Span", ScriptParameterType.Integer, chikouspan.ToString()),
                new ScriptParameter("Buy Signal Strength", ScriptParameterType.String, buySignalStrength),
				new ScriptParameter("Sell Signal Strength", ScriptParameterType.String, sellSignalStrength)
            };
            return res;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
        	tenkansen = ParseInt(parameters["Tenkan Sen"], tenkansen);
        	kijunsen = ParseInt(parameters["Kijun Sen"], kijunsen);
        	senkouspanb = ParseInt(parameters["Senkou Span B"], senkouspanb);
        	chikouspan = ParseInt(parameters["Chikou Span"], chikouspan);
        	buySignalStrength = ParseString(parameters["Buy Signal Strength"], buySignalStrength);
			sellSignalStrength = ParseString(parameters["Sell Signal Strength"], sellSignalStrength);
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, LineType = 1, Name = "LeadSpanA", HexLineColor = "#88FF88" },
                new ChartRecord() { ChartIndex = 0, LineType = 1, Name = "LeadSpanB", HexLineColor = "#FF8888" },
                new ChartRecord() { ChartIndex = 0, Name = "ConvLine", HexLineColor = "#0066FF" },
                new ChartRecord() { ChartIndex = 0, Name = "BaseLine", HexLineColor = "#FF0000" },
                new ChartRecord() { ChartIndex = 0, Name = "LagSpan", HexLineColor = "#00FF00" }
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                Convert.ToDecimal(leadingSpanA),
                Convert.ToDecimal(leadingSpanB),
                Convert.ToDecimal(convLine),
                Convert.ToDecimal(baseLine),
                Convert.ToDecimal(laggingSpan)
            };

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
            // In this code example we do not have to do anything
        }

        // This is the brain so the say, this is where the calculation is done and an output must be produced.
        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
				var position = instrument.Close.Length - (context.Offset + 1);
				var price = instrument.Close[position];
				var senkouPrice = instrument.Close[position - senkouspanb];
				var chikouPrice = instrument.Close[position];
				
				// Conversion Line
				var temp1 = instrument.Close.Skip(position - tenkansen).Take(tenkansen);
				convLine = (temp1.Max() + temp1.Min()) / 2.0;
				
				// Base Line
				var temp2a = instrument.High.Skip(position - kijunsen).Take(kijunsen);
                var temp2b = instrument.Low.Skip(position - kijunsen).Take(kijunsen);
                baseLine = (temp2a.Max() + temp2b.Min()) / 2.0;
				
				// Lagging Span
                if(position < instrument.Close.Length - chikouspan)
                {
                    laggingSpan = instrument.Close[position + chikouspan];
                }

				// Leading Span A
				var temp4a = instrument.Close.Skip(position - tenkansen - chikouspan).Take(tenkansen);
				var temp4b = (temp4a.Max() + temp4a.Min()) / 2.0; //convLine moved by chikou setting value
				var temp4c = instrument.High.Skip(position - kijunsen - chikouspan).Take(kijunsen);
                var temp4d = instrument.Low.Skip(position - kijunsen - chikouspan).Take(kijunsen);
                var temp4e = (temp4c.Max() + temp4d.Min()) / 2.0; //baseLine moved by chikou setting value
				leadingSpanA = (temp4b + temp4e) / 2.0;
				
				// Leading Span B
				var temp3a = instrument.High.Skip(position - senkouspanb - chikouspan).Take(senkouspanb);
                var temp3b = instrument.Low.Skip(position - senkouspanb - chikouspan).Take(senkouspanb);
                leadingSpanB = (temp3a.Max() + temp3b.Min()) / 2.0;
								
				var leadingBelow = Math.Min(leadingSpanA, leadingSpanB);
				var leadingAbove = Math.Max(leadingSpanA, leadingSpanB);

                var minSpan = Math.Min(leadingSpanA, leadingSpanB);
                var maxSpan = Math.Max(leadingSpanA, leadingSpanB);

				// Bullish Tenkan Kijun Cross, Chikou over Price, Price over Kijun, Kumo Breakout
				if (convLine > baseLine && chikouPrice > instrument.Close[position - chikouspan] && instrument.Close[position] > baseLine && instrument.Close[position] > maxSpan)
				{
					// convline and baseline can be crossed everywhere - a really weak signal
					if (buySignalStrength.Contains(WEAK))
						return IndicatorResult.BuyLong;
					
					// convline and baseLine inside or above kumo - a neutral signal and Kumo Breakout
					else if (buySignalStrength.Contains(NEUTRAL) && convLine > minSpan && baseLine > minSpan)
						return IndicatorResult.BuyLong;

					// convline and baseLine above kumo - a strong signal and Kumo Breakout
					else if (buySignalStrength.Contains(STRONG) && convLine > maxSpan && baseLine > maxSpan)
						return IndicatorResult.BuyLong;
				}

				// Bearish TK Cross, Chikou below price, Price below Kijun
				else if (convLine < baseLine && chikouPrice < instrument.Close[position - chikouspan] && instrument.Close[position] < baseLine)
				{
					// convline above kumo
					if (sellSignalStrength.Contains(WEAK))
						return IndicatorResult.SellShort;

					// convline inside kumo
					else if (sellSignalStrength.Contains(NEUTRAL) && convLine < maxSpan)
						return IndicatorResult.SellShort;

					// convline below kumo
					else if (sellSignalStrength.Contains(STRONG) && convLine < minSpan)
						return IndicatorResult.SellShort;
				}		
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }
        
        
        
        int ParseInt(object value, int lastValue)
	    {
	    	try {
	    		return Convert.ToInt32(value.ToString());
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    double ParseDouble(object value, double lastValue)
	    {
	    	var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			try {
				double output;
				double.TryParse(
					value.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
				return output;
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    string ParseString(object value, string lastValue)
	    {
	    	try {
	    		return Convert.ToString(value);
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
    }
}
    